/***************************************************************************
                          ksmoothdock.h  -  description
                             -------------------
    begin                : Tue Jun 10 22:18:34 BST 2003
    copyright            : (C) 2003-2005 by KSmoothDock team
    email                : dangvd@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSMOOTHDOCK_H
#define KSMOOTHDOCK_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kpopupmenu.h>
#include <kwinmodule.h>
#include <kaboutdialog.h>
#include <krootpixmap.h>
#include <kpixmap.h>
#include <ksharedpixmap.h>

#include <qwidget.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <qcolor.h>
#include <qdatetime.h>

#include <netwm.h>

#include "constants.h"
#include "item.h"

class ConfDialog;
class ConfMenuDialog;
class AboutDialog;
class ToolTip;
class KConfig;

/** KSmoothDock is the base class of the project */
class KSmoothDock : public QWidget
{
  Q_OBJECT
  public:
  
    /** construtor */
    KSmoothDock(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~KSmoothDock();

    // methods to get and set configuration variables

    int getZoomMode(); // zoom mode is read-only
    int getAnimSpeedNZ();
    int getAnimSpeedPZ();
    
    int getGradualPZ();
    int getShowPager();
    int getShowTaskbar();
    int getShowWinAllDesks(); // show windows from all desktops
    int getShowClock();    
    int getShowNotification(); // show-notification flag
    int getShowBorders(); // show-borders flag
    QColor getBorderColor();    
    
    int getSmallIconSizeNZ(); // normal zoom
    int getBigIconSizeNZ();
    int getBaseIconSizeNZ();

    int getSmallIconSizePZ(); // parabolic zoom
    int getBigIconSizePZ();
    int getBaseIconSizePZ();

    int getAutohide();

    int getDockOpacity();
    QColor getBgColor();

    QColor getSepColor(); // separator bar's color

    int getScrWidth(); // fix for xinerama

    QColor getActiveDeskColor();
    QColor getInactiveDeskColor();
		
			// tooltip
		
		int getShowToolTip();	
		QString getToolTipFontFace();
		int getToolTipFontSize();		
		int getToolTipFontIsItalic();
		int getToolTipFontIsBold();
		QColor getToolTipFontColor();
		QColor getToolTipFontBgColor();
		
			// clock
		
		int getUse24HourClock();
		QColor getClockFontColor();
		QString getClockFontFace();
		int getClockFontSize();
		int getClockFontIsItalic();
		int getClockFontIsBold();

    void setGradualPZ(int val);
    void setAnimSpeedNZ(int val);
    void setAnimSpeedPZ(int val);
    
    void setShowPager(int val);
    void setShowTaskbar(int val);
    void setShowWinAllDesks(int val); // show windows from all desktops
    void setShowClock(int val);
		
    void setShowNotification(int val); // show-notification flag
    void setShowBorders(int val); // show-borders flag
    void setBorderColor(QColor val);

    //void setPosition(PanelPosition val);

    void setSmallIconSizeNZ(int val); // normal zoom
    void setBigIconSizeNZ(int val);
    void setBaseIconSizeNZ(int val);

    void setSmallIconSizePZ(int val); // parabolic zoom
    void setBigIconSizePZ(int val);
    void setBaseIconSizePZ(int val);

    void setAutohide(int val);

    void setDockOpacity(int val);
    void setBgColor(QColor val);

    void setSepColor(QColor val); // separator bar's color

    void setScrWidth(int val); // fix for xinerama

    void setActiveDeskColor(QColor val);
    void setInactiveDeskColor(QColor val);

			// tooltip
		
		void setShowToolTip(int val);	
		void setToolTipFontFace(QString val);
		void setToolTipFontSize(int val);		
		void setToolTipFontIsItalic(int val);
		void setToolTipFontIsBold(int val);
		void setToolTipFontColor(QColor val);
		void setToolTipFontBgColor(QColor val);
		
			// clock
		
		void setUse24HourClock(int val);
		void setClockFontColor(QColor val);
		void setClockFontFace(QString val);
		void setClockFontSize(int val);
		void setClockFontIsItalic(int val);
		void setClockFontIsBold(int val);

    void loadConf();
    void saveConf();
    void saveConf(int zoomMode);

    void setStateBelow();
    void setStateAbove();

    void updateIconSizes();
    void updateComponents();
    void updateScrWidth();
	void updateTooltips();
	void initAutohideMode();
	
	void updateAnimSpeedNZ();
    	void updateAnimSpeedPZ();

  public slots:
    void edit();
    void editPref();
    void about();
    void switchZoomMode();
		
		void updateQuickLaunchers();

    void windowAdded(WId id);
    void windowRemoved(WId id);
    void windowChanged(WId id, unsigned int properties);
    void activeWindowChanged(WId id);
    void currentDesktopChanged(int desktop);

//    void enterTimerTicked();
    void onChangeTimerTicked();
    void timerTicked(); // normal mode
		void timerTickedPZ(); // parabolic mode
    void clockTicked();

    void updateBackground(const QPixmap&);
    void updateBackground();
    void captureAndUpdateBackground();
    void captureBackground();

    void wallpaperLoaded(bool);

    void minApp();
    void maxApp();
    void restApp();
    void closeApp();
  
	protected:
    
    void paintEvent(QPaintEvent *);
    void mousePressEvent( QMouseEvent * );
    void mouseMoveEvent(QMouseEvent* );
    void enterEvent(QEvent*);
    void leaveEvent(QEvent*);
				  	
	private:
    
    void run(QString cmd);
    int func(int x);
    void doUpdateGeometry();
    void addTask(WId id);

    void loadMenu();
    void addPager();
    void loadTasks();

    void addMainPopupMenu();
    void addAboutDlg();

    void showDesktop();  
		
		void setKickerPosition();  

    void doMinimise();

    void showWindowStack();

    void loadWallpaper();
		
		void paintHiddenMode();
		void updateBackgroundHiddenMode(const QPixmap&);
		void mouseMoveEventHiddenMode(QMouseEvent *);

    void paintEventPZ(QPaintEvent *);
    void mousePressEventPZ( QMouseEvent * );
    void mouseMoveEventPZ(QMouseEvent* );
    void enterEventPZ(QEvent*);
    void leaveEventPZ(QEvent*);
    void doUpdateGeometryPZ();
    void updateBackgroundPZ(const QPixmap&);
    void updateBackgroundPZ();

    void paintEventNZ(QPaintEvent *);
    void mousePressEventNZ( QMouseEvent * );
    void mouseMoveEventNZ(QMouseEvent* );
    void enterEventNZ(QEvent*);
    void leaveEventNZ(QEvent*);
    void doUpdateGeometryNZ();
    void updateBackgroundNZ(const QPixmap&);
    void updateBackgroundNZ();

    int getIndex(int x); // normal mode only    

  private slots:
    void stopTaskLauncherNotification();
		void reenableParabolicZoom();
		
		void doShow();
		void doHide();
		
		void continueZoomOut();
		
  private:

    // configurable variables

      //  panel components
        
    int fShowPager;
    int fShowTaskbar; // show-taskbar flag
    int fShowWinAllDesks; // show windows from all desktops
    int fShowClock;

      // icon zooming

    int zoomMode; // zoom mode is read-only
    int fGradualPZ; // gradual flag in parabolic zoom mode
    int animSpeedNZ; // zoom animation speed: 1 ... 31 (normal zoom)
    int animSpeedPZ; // zoom animation speed: 1 ... 31 (parabolic zoom)

      // arrangement

    //PanelPosition position;
    int smallIconSizeNZ; // normal zoom icon sizes
    int bigIconSizeNZ;
    int baseIconSizeNZ;

    int smallIconSizePZ; // parabolic zoom icon sizes
    int bigIconSizePZ;
    int baseIconSizePZ;

      // hiding

    int fAutohide;

      // appearance

    int fShowBorders; // show-borders flag
    QColor borderColor;
    QColor bgColor;
    QColor sepColor; // separator bar's color
    QColor activeDeskColor;
    QColor inactiveDeskColor;

      // misc
      
    int fShowNotification; // show-notification flag

    int dockOpacity;
    int scrWidth; // fix for xinerama

			// tooltip
		
		int fShowToolTip;	
		QString ttFontFace;
		int ttFontSize;		
		int fTFIsItalic;
		int fTFIsBold;
		QColor ttFontColor;
		QColor ttBgColor;
		
			// clock
		
		int fUse24HourClock;
		QColor clockFontColor; // clock text color
		QString clockFontFace;
		int clockFontSize;		
		int fCFIsItalic;
		int fCFIsBold;
		
		// end configurable variables

    // normal variables

    ConfDialog *confDlg; // configuration dialog
		ConfMenuDialog *confMenuDlg; // quick launch menu configuration dialog

    int smallIconSize;
    int bigIconSize;
    int baseIconSize;

    QString progPath;
    QString menuPath;
    QString confFile;


    NETWinInfo *info;
    NETRootInfo *rootinfo;
    KWinModule *wm;

    // QWidget *supportWin;

    WindowMode wMode;
    bool isActive;

    int numLaunchers; // number of quick launcher items

    int h0, h1, w0, x0, w1, x1, deltaX, deltaH; // parabolic zooming mode
    int w_cur; // current width - gradual zooming - parabolic zooming mode 
    int h, w, x; // normal zooming mode
		int hw, hh; // hidden mode width and height

    int ix, iy, iDist, iSpace;
    int dw, dh;
    int funcW, funcH;
    int ii_first, ii_last;
    int last_ii_first, last_ii_last;
    int ii_updating_first, ii_updating_last;
    int sx, sw;
    int adjust;
    int last_mx;

    // normal mode

    QTimer *animTimer;
    int iSizeInc[MAX_ICONS];

    // common
		
		KConfig *config;

    QPtrList<Item> items;
    int iSize[MAX_ICONS + 1];
    int cx[MAX_ICONS + 1];
    int cur_cx[MAX_ICONS + 1];
    QPixmap offscr;
		QString newCommand; // the new command to launch (in launcher click event)

    KRootPixmap *rootpix;
    KPixmap bottomBg; // faded bottom background
    QPixmap bottomBgOrg; // original bottom background
    QPixmap topBg;    
		KPixmap hiddenBg; // background image (hidden mode)

    KPopupMenu *popup;
    KPopupMenu *appMenu;
    AboutDialog *aboutDlg;
//    QTimer *enterTimer; // used in enterEvent
    QTimer *onChangeTimer; // used when a window has changed and we need to notify user

    int animStep; // animation step, not in use yet
    int iOnClick; // index of the quick launcher being clicked
    int iOnChange; // index of the task whose window has changed
    WId idOnChange; // WId of the task whose window has changed
    WId cId;  // id of the application (used in the right click menu over the taskbar)
    WId activeId; // id of the active window
		WId clickedId; // id of the clicked running task
    int onChangeAnimStep;

    bool fJustEntered;
//    bool fDesktopChanged;
//    bool fJustLeft;
//    bool fDrawAsJustEntered;

    // pager variables

    // common

    int curDesk; // current desktop
    int numDesks;
		int newDesk; // the new desktop (in pager click event)

    KSharedPixmap *wallpapers[MAX_DESKTOPS];
		bool fWallpapersReady[MAX_DESKTOPS];

    // Normal mode only

    int pagerDeskW;
    int pagerDeskH;
    QPixmap *tnWallpapers[MAX_DESKTOPS];
    int pagerW;

    // clock

    int clockW;
    QTimer *clockTimer;
    bool fUpdateClock;
		
		// tool tip
		
		ToolTip *toolTipWin;
		int curInd;
		int prevInd;
		
		// for gradual zoom-in and zoom-out
		
		int cx_start[MAX_ICONS + 1];
		int cx_end[MAX_ICONS + 1];
		int iSize_start[MAX_ICONS + 1];
		int iSize_end[MAX_ICONS + 1];
        int w_start; // starting width of the dock in the animation process
        int w_end; // ending width of the dock in the animation process
		QTimer *animTimerPZ;
		int animStepPZ;
		bool isIncreasing;
		bool taskClicked;
		bool pagerClicked;
		bool launcherClicked;
    bool showDeskClicked;
    
    // Zoom animation
    	// Normal zoom only
    
    int anim_num_step_NZ; // number of steps
    int anim_step_delay_NZ; // delay time between step
    
    	// Parabolic zoom only
    
    int anim_num_step_PZ; // number of steps
    int anim_step_delay_PZ; // delay time between step
    
    // Variales dealing with wallpaper changes (for example, slideshow wallpapers)
    
    QString deskConfFile; // path to KDE's desktop configuration file
    QDateTime lastDeskConfFileUpdated; // the time when the above file is last updated
};

#endif
